/*_____________________________________________________________________________________________________________________________________________________________________
SNS_#TABLE.mq4

Copyright (C) 2009, Steven N. Scott
mailto:steven.n.scott@gmail.com
_______________________________________________________________________________________________________________________________________________________________________


														DEMONSTRATION OF SNS_TABLE_OBJECT USAGE

_______________________________________________________________________________________________________________________________________________________________________*/ 


#property copyright "Copyright(c) 2009, Steven N. Scott"
#property link      "mailto:steven.n.scott@gmail.com"

// The following INCLUDE HEADER FILES need to be in the EXPERTS\INCLUDES Folder
#include <SNS_TABLE_OBJECT.mqh>
#include <SNS_ObjectLibrary.mqh>

#property indicator_chart_window

//=====================================================================================================================================================================
//=====================================================================================================================================================================
//=====================================================================================================================================================================
//=====================================================================================================================================================================
//=====================================================================================================================================================================




//____________________________________________________________________________________________________________________________________________________________
// INDICATOR CUSTOMIZABLE PROPERTIES
//	By convention (only), I use row/column 0/0 (corner=0) as the place to display a table header/name and ignore the remaining cells on row 0.
// You do not have to follow this convention and can use row 0 for data to have a table with no separate "header" on top.
// This example uses row 0 to display the a header of just the hard-coded phrase "TABLE TITLE".
//
// Simple way to re-use this is to copy the following lines to your indicator and also copy over the TABLE.INITIALIZE function to be called from your init function.


#define ROWS 3
#define COLUMNS 3

extern string 		Table.Title								=	"TestTable";

extern int			Table.Window							=  0;
extern int			Table.Corner							=  0;
extern int			Table.X									=	10;
extern int			Table.Y									=  20;
extern bool			Table.Back								=  false;

						// List Column (X) and Row (Y) values as string with values separated by semi-colons
extern string		Table.Columns 							=	"10;230;430";
extern string		Table.Rows 								=  "10;40;60";

						// List cell attributes as a string with values separated by semi-colons
extern string		Cell.Fonts 								=	"Verdana Bold;Arial;Arial;Arial;Tahoma;Verdana;Courier New;Arial;Verdana";			
extern string		Cell.FontSizes							=	"18;0;0;14;12;12;18;10;10";	
extern string		Cell.FontColors						=  "Orange;Orange;Orange;Red;White;Blue;Lime;Yellow;lightskyblue";	



//____________________________________________________________________________________________________________________________________________________________
//INTERNAL GLOBALS

bool Table.Initialized=false;

string tag;
string temp[];

int row.count,col.count,table.col[],table.row[];



//************************************************************************************************************************************************************
//STANDARD INDICATOR FUNCTIONS

int init(){

	tag=Table.Title;

	Table.Initialize(tag);
	
   return(0);

}


int deinit() {

	Table.Deinit();
   DeleteObjectsByPrefix(tag);
   Comment("");
   return(0);
   
}

int start(){

   deinit();

   TestTable();

	int lasterr = GetLastError();
	if (lasterr!=0) Print("ERROR " + lasterr);

   return(0);
   
}



//***************************************************************************************************************************************************************
//CUSTOM INDICATOR FUNCTIONS


//____________________________________________________________________________________________________________________________________________________________
//INITIALIZE THE TABLE OBJECT USING EXTERN PARAMETERS PROVIDED BY *THIS* (YOUR) INDICATOR 

void Table.Initialize(string name){

	string f[],fonts[ROWS][COLUMNS],fcolors[];
	color colors[ROWS][COLUMNS];
	int cols[],rows[],fsizes[],sizes[ROWS][COLUMNS];
	
	String.ToIntArray(cols,Table.Columns);
	String.ToIntArray(rows,Table.Rows);
	
	Table.SetCorner(Table.Corner);
		
	if (Table.Init(name,rows,cols)<0) Print("TABLE FAILED TO INITIALIZE"); else Table.Initialized=true;

	// convert EXTERN string "arrays" into "real" arrays (because MT4 does not provide for EXTERN of array variables)
	String.ToArray(f,Cell.Fonts);
	String.ToIntArray(fsizes,Cell.FontSizes);
	String.ToArray(fcolors,Cell.FontColors);
	
	// All of the following array setting shenanigans are because of the peculiarities of MQL4's implementation of multi-dimensional arrays
	// BUT, doing it this way also makes it visually very clear what each cell property is being set to.
	
	int i=0;
	for (int r=0; r<ROWS; r++) {
	
		for(int c=0; c<COLUMNS; c++) {
		
			fonts[r,c]=f[i];
			sizes[r,c]=fsizes[i];
			colors[r,c]=String.ToColor(fcolors[i]);
			i++;
		
		}
	
	
	}	
	//EXAMPLE: Offset cell[2,1]'s row (Y coord) to make it line up better with larger text in cell[2,0]
	Table.SetCellOffsets(2,1,10,0);
		
	Table.SetFonts(fonts);
	Table.SetFontColors(colors);
	Table.SetFontSizes(sizes);

	// Now run through cells and find any that were not set (missing from EXTERN string) and set their properties to these DEFAULT values:
	Table.SetDefaultFont("Verdana",12,White);
	
	//Now the table is fully initialized and is ready to have data/text stuffed into each cell by the rest of your indicator code

}


//____________________________________________________________________________________________________________________________________________________________
//SIMPLE TEST OF SETTING THE TEXT IN EACH OF THE AVAILABLE TABLE CELLS
//Cell[2,1] demonstrates the use of CELL.OFFSET.ROW to move text down in individual cell (which may be necessary if other text on that ROW is LARGER)

void TestTable(){

	if(Table.Initialized) {
	
		Table.SetText(0,0,"TABLE TITLE");
		
		Table.SetSymbol(Symbol());
		Table.SetText(1,1,"Tahoma 12");
		Table.SetText(1,2,"Verdana 12");
		
		Table.SetPeriod(Period());
		Table.SetText(2,1,"Arial 10 (cell's row offset by +10)");		
		Table.SetText(2,2,"Verdana 10 (no row offset)");		

	}
	
}


//____________________________________________________________________________________________________________________________________________________________
//EXAMPLE OF FUNCTIONS THAT "KNOW" WHICH CELL TO WRITE DATA TO

void Table.SetSymbol(string symbol){
		// Put current SYMBOL into cell[1,0]
		Table.SetText(1,0,Symbol()); 

}

void Table.SetPeriod(int period){
		// Put current TIME FRAME into cell[2,0]
		Table.SetText(2,0,Time.TFToString(period));

}