/*___________________________________________________________________________________________________________________________________________________________________
SNS_TABLE_OBJECT.mq4 

SNS DEVELOPER TOOLS
Copyright (C) 2009, Steven N. Scott
mailto:steven.n.scott@gmail.com
_________________________________________________________________________________________________________________________________________________________________
Version History

v1.0[BETA]			Beta Release																							11/30/2009
v1.0a					Fixed bug with location of mqh file																12/03/2009


_________________________________________________________________________________________________________________________________________________________________

TABLE OBJECT

A tool for MQL4 developers to make it easier to create a grid of labels on a chart with easy access control of fonts, colors, and sizes as well
as the text that goes into each cell.  Developer can specify the X,Y coordinates of each cell using the SetColumns and SetRows methods.


_______________________________________________________________________________________________________________________________________________________________________*/ 


#property copyright "Steven N. Scott"
#property link      ""
#property library

//The SNS_OBJECTLIBRARY.MQH INCLUDE/HEADER FILE NEEDS TO BE IN THE EXPERTS/INCLUDES FOLDER for TABLE_OBJECT TO FIND TI
#include <SNS_ObjectLibrary.mqh>


//=====================================================================================================================================================================
//GLOBALLY EXPOSED PROPERTIES (VARIABLES WITH GETTERS AND SETTERS BELOW)
// Note, in Object Oriented Programming, you do not allow users of the object to directly set or read properties - you access them through
// "getter" and "setter" methods as seen below.


 string 		Table.Title								=	"SNSTABLE";

 int			Table.Window							=  0;
 int			Table.Corner							=  0;
 
 //			Top Left corner of table relative to designated CORNER
 int			Table.X									=	10;
 int			Table.Y									=  20;
 
 // 			Should cells/labels be drawn as "background" objects?
 bool			Table.Back								=  false;
 
 //			Default of 3 columns and 2 rows
 int			Table.Columns[100] 					=	{10,100,200};
 int			Table.Rows[100]						=  {10,30};
 
 string		Table.DefaultFont						= 	"Arial";
 int			Table.DefaultSize						=	12;
 color		Table.DefaultColor					=	DimGray;

 //			If you have a table with more than 100 columns and rows, you have a problem :-)
 string		Cell.Font[100][100];
 int			Cell.FontSize[100][100]; 			
 color		Cell.FontColor[100][100];	
 
 int			Cell.Offset.COL[100][100];
 int			Cell.Offset.ROW[100][100];	


//____________________________________________________________________________________________________________________________________________________________
//INTERNAL GLOBALS

bool Table.Initialized=false;

string table.error;
string temp[];

int row.count,col.count;

string cell.text[][100];

//____________________________________________________________________________________________________________________________________________________________
//PROPERTY GETTERS AND SETTERS
// These methods simulate Object Oriented programming where properties of an object are acessed through getter and setter methods rather than manipulated
// directly - allows you to "make things happen" when a property is changed or accessed.

// TITLE is important as it allows you to create/iuse multiple table objects on one chart as long as each has a different TITLE
void 		Table.SetTitle(string title) { Table.Title=title; }
string 	Table.GetTitle() { return(Table.Title); }

// With SETWINDOW, you can force the table to display on any available window in your template (default is window 0, or the main chart window)
void 		Table.SetWindow(int win) { Table.Window=win; }
int 		Table.GetWindow() { return(Table.Window); }

// Note that setting CORNER to other than 0 (top left) affects how the other X/Y coordinates are used by MT4 (which can be a royal pain)
// For *ME*, it is easier and more intuitive to just leave corner as 0 (upper left) and space everything from there
void 		Table.SetCorner(int corner) { Table.Corner=corner; }
int 		Table.GetCorner() { return(Table.Corner); }

void		Table.SetX(int x) { Table.X=x; }
int		Table.GetX(int x) { return(Table.X); }
void		Table.SetY(int y) { Table.Y=y; }
int		Table.GetY(int y) { return(Table.Y); }
void		Table.SetXY(int x, int y) {Table.X=x; Table.Y=y; }
void 		Table.SetColumns(int cols[]) {	ArrayCopy(Table.Columns,cols); }
void 		Table.SetRows(int rows[]) {	ArrayCopy(Table.Rows,rows); }

//			Font values can be set all at once using an array and the SETxxxS method, or individually using SETFONTxxx.
void 		Table.SetFonts(string fonts[][]) { for(int r=0; r<row.count; r++) for (int c=0; c<col.count; c++) Cell.Font[r,c]=fonts[r,c]; }
void 		Table.SetFont(int row, int col, string font) { Cell.Font[row,col]=font; }
string 	Table.GetFont(int row, int col) { return(Cell.Font[row,col]); }

void 		Table.SetFontColors(color colors[][]){ for(int r=0; r<row.count; r++) for (int c=0; c<col.count; c++) Cell.FontColor[r,c]=colors[r,c];}
void		Table.SetFontColor(int row, int col, color clr) { Cell.FontColor[row,col]=clr; }
color		Table.GetFontColor(int row, int col) { return(Cell.FontColor[row,col]); }

void 		Table.SetFontSizes(int sizes[][]){ for(int r=0; r<row.count; r++) for (int c=0; c<col.count; c++) Cell.FontSize[r,c]=sizes[r,c];}
void		Table.SetFontSize(int row, int col, int size) { Cell.FontSize[row,col]=size; }
int		Table.GetFontSize(int row, int col) { return(Cell.FontSize[row,col]); }

void		Table.SetCellOffsets(int row, int col, int OffRow, int OffCol) { Cell.Offset.ROW[row,col]=OffRow; Cell.Offset.COL[row,col]=OffCol; }
int		Table.GetCellOffsetCol(int row, int col) { return(Cell.Offset.COL[row,col]); }
int		Table.GetCellOffsetRow(int row, int col) { return(Cell.Offset.ROW[row,col]); }

void 		Table.SetDefaultFont(string name, int size, color clr) { 

	Table.DefaultFont=name; Table.DefaultSize=size; Table.DefaultColor=clr; 
	for(int r=0; r<row.count; r++) for(int c=0; c<col.count; c++) { 
	
		if(Cell.Font[r,c]=="") Cell.Font[r,c]=name; 
		if(Cell.FontSize[r,c]==0) Cell.FontSize[r,c]=size; 
		if(Cell.FontColor[r,c]==CLR_NONE) Cell.FontColor[r,c]=clr; 
		
	}
	
}

//*************************************************************************************************************************************************************
//OBJECT FUNCTIONS
// These methods are generally how you use the table within your MQL indicator.


//____________________________________________________________________________________________________________________________________________________________
//"INITIALIZES" THE TABLE OBJECT BUT DOES NOT ACTUALLY "create" any objects at this time

int Table.Init(string TableName, int rows[], int cols[]){

	int i,lasterr;
	Table.SetColumns(cols);
	Table.SetRows(rows);
	col.count=ArraySize(Table.Columns);
	row.count=ArraySize(Table.Rows);
	if(col.count==0 || row.count==0) Alert("ERROR");
	
	if(String.Trim(TableName)!="") Table.SetTitle(TableName);
	return(0);
	
}


//____________________________________________________________________________________________________________________________________________________________
//HANDLES CLEANING UP OBJECTS CREATED BY THE TABLE OBJECT ITSELF

int Table.Deinit() {

   DeleteObjectsByPrefix(Table.Title);
   return(0);
   
}

void Table.SetTextOverride(int row, int col, string text, string font="", int size=0, color clr=CLR_NONE){

	cell.text[row,col]=text;
	Table.DrawCell(row,col,font,size,clr);


}


void Table.SetText(int row, int col, string text){

	cell.text[row,col]=text;
	Table.DrawCell(row,col,"",0,CLR_NONE);


}

void Table.DrawCell(int row, int col, string font, int size, color clr=CLR_NONE) {

	string cell="("+row+":"+col+")";
	if(font=="") font=Cell.Font[row,col];
	if(size==0) size=Cell.FontSize[row,col];
	if(clr==CLR_NONE)	clr =Cell.FontColor[row,col];
	Draw_Label(Table.Title, cell, cell.text[row,col], Table.Corner, Table.X+Table.Columns[col]+Cell.Offset.COL[row,col], Table.Y+Table.Rows[row]+Cell.Offset.ROW[row,col], font, size, clr, Table.Back, Table.Window);
	
	//Print(row+","+col+"  [" + Cell.Font[row,col]+"]");

}



